import React from 'react';
import { EditorView } from '@codemirror/view';
import { Compartment, EditorState } from '@codemirror/state';
export const useCodeMirror = (containerRef, onChange, value = '', extensions) => {
  const lastValueRef = React.useRef(value);
  const onChangeRef = React.useRef(undefined);
  const [editor, setEditorView] = React.useState(undefined);
  const readOnlyCompartment = React.useRef(new Compartment());
  onChangeRef.current = React.useMemo(() => onChange, [onChange]);
  React.useEffect(() => {
    const instanceMounted = {
      current: 0
    };
    const editorViewInternal = new EditorView({
      state: EditorState.create({
        extensions: [...(extensions || []), EditorView.updateListener.of(viewUpdate => {
          if (instanceMounted.current === 0) {
            instanceMounted.current = 1;
            return;
          }

          if (instanceMounted.current !== 1) {
            return;
          }

          const next = viewUpdate.state.doc.toString();
          const prev = lastValueRef.current;
          lastValueRef.current = next;
          const onChange = onChangeRef.current;

          if (!onChange || viewUpdate.view.destroyed) {
            return;
          }

          onChange(viewUpdate, next, prev);
        }), readOnlyCompartment.current.of(EditorView.editable.of(Boolean(onChangeRef.current)))],
        doc: lastValueRef.current
      }),
      parent: containerRef.current
    });
    setEditorView(editorViewInternal);
    return () => {
      instanceMounted.current = 2;
      editorViewInternal === null || editorViewInternal === void 0 ? void 0 : editorViewInternal.destroy();
      setEditorView(undefined);
    };
  }, [containerRef, lastValueRef, setEditorView, extensions, onChangeRef]);
  React.useEffect(() => {
    if (!editor) {
      return;
    }

    editor.dispatch({
      effects: readOnlyCompartment.current.reconfigure(EditorView.editable.of(Boolean(onChange)))
    });
  }, [editor, onChange]);
  React.useEffect(() => {
    if (containerRef.current && lastValueRef.current !== value) {
      setEditorView(editor => {
        if (lastValueRef.current === value) {
          return editor;
        }

        editor === null || editor === void 0 ? void 0 : editor.dispatch({
          changes: {
            from: 0,
            to: editor === null || editor === void 0 ? void 0 : editor.state.doc.length,
            insert: value
          }
        });
        lastValueRef.current = value;
        return editor;
      });
    } else {
      lastValueRef.current = value;
    }
  }, [containerRef, value, setEditorView]);
  return [editor, setEditorView];
};