"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useCodeMirror = void 0;

var _react = _interopRequireDefault(require("react"));

var _view = require("@codemirror/view");

var _state = require("@codemirror/state");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var useCodeMirror = function useCodeMirror(containerRef, onChange) {
  var value = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '';
  var extensions = arguments.length > 3 ? arguments[3] : undefined;

  var lastValueRef = _react["default"].useRef(value);

  var onChangeRef = _react["default"].useRef(undefined);

  var _React$useState = _react["default"].useState(undefined),
      _React$useState2 = _slicedToArray(_React$useState, 2),
      editor = _React$useState2[0],
      setEditorView = _React$useState2[1];

  var readOnlyCompartment = _react["default"].useRef(new _state.Compartment());

  onChangeRef.current = _react["default"].useMemo(function () {
    return onChange;
  }, [onChange]);

  _react["default"].useEffect(function () {
    var instanceMounted = {
      current: 0
    };
    var editorViewInternal = new _view.EditorView({
      state: _state.EditorState.create({
        extensions: [].concat(_toConsumableArray(extensions || []), [_view.EditorView.updateListener.of(function (viewUpdate) {
          if (instanceMounted.current === 0) {
            instanceMounted.current = 1;
            return;
          }

          if (instanceMounted.current !== 1) {
            return;
          }

          var next = viewUpdate.state.doc.toString();
          var prev = lastValueRef.current;
          lastValueRef.current = next;
          var onChange = onChangeRef.current;

          if (!onChange || viewUpdate.view.destroyed) {
            return;
          }

          onChange(viewUpdate, next, prev);
        }), readOnlyCompartment.current.of(_view.EditorView.editable.of(Boolean(onChangeRef.current)))]),
        doc: lastValueRef.current
      }),
      parent: containerRef.current
    });
    setEditorView(editorViewInternal);
    return function () {
      instanceMounted.current = 2;
      editorViewInternal === null || editorViewInternal === void 0 ? void 0 : editorViewInternal.destroy();
      setEditorView(undefined);
    };
  }, [containerRef, lastValueRef, setEditorView, extensions, onChangeRef]);

  _react["default"].useEffect(function () {
    if (!editor) {
      return;
    }

    editor.dispatch({
      effects: readOnlyCompartment.current.reconfigure(_view.EditorView.editable.of(Boolean(onChange)))
    });
  }, [editor, onChange]);

  _react["default"].useEffect(function () {
    if (containerRef.current && lastValueRef.current !== value) {
      setEditorView(function (editor) {
        if (lastValueRef.current === value) {
          return editor;
        }

        editor === null || editor === void 0 ? void 0 : editor.dispatch({
          changes: {
            from: 0,
            to: editor === null || editor === void 0 ? void 0 : editor.state.doc.length,
            insert: value
          }
        });
        lastValueRef.current = value;
        return editor;
      });
    } else {
      lastValueRef.current = value;
    }
  }, [containerRef, value, setEditorView]);

  return [editor, setEditorView];
};

exports.useCodeMirror = useCodeMirror;